// cf. https://blog.thoughtram.io/angular/2016/07/27/custom-form-controls-in-angular-2.html
"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var jalhyd_1 = require("jalhyd");
var param_service_1 = require("../../services/param/param.service");
var internationalisation_service_1 = require("../../services/internationalisation/internationalisation.service");
var ParamInputComponent = ParamInputComponent_1 = (function () {
    function ParamInputComponent(paramService, changeDetector, intlService) {
        this.paramService = paramService;
        this.changeDetector = changeDetector;
        this.intlService = intlService;
        /**
         * flag d'affichage du titre
         */
        this.displayTitle = false;
        // ControlValueAccessor interface
        this.propagateChange = function (_) { };
        this._id = ParamInputComponent_1._idGen++;
        if (ParamInputComponent_1._startTime == undefined)
            ParamInputComponent_1._startTime = new Date().getTime();
        this._uiValue = new jalhyd_1.NumericalString();
    }
    ParamInputComponent.prototype.hasError = function () {
        var res = (this._message != undefined);
        if (res)
            this.log("hasError : true " + this._message);
        else
            this.log("hasError : false");
        return res;
    };
    ParamInputComponent.prototype.getSfromUI = function () {
        return this._fromUI ? " fromUI " : " fromMODEL";
    };
    ParamInputComponent.prototype.getSParam = function () {
        return " " + this._paramDef.symbol + "=" + this._paramDef.toString();
    };
    ParamInputComponent.prototype.getSUIvalue = function (v) {
        if (v === void 0) { v = undefined; }
        if (v == undefined)
            return "  uiValue=" + this._uiValue.toString() + "";
        return "  uiValue=" + v.toString() + "";
    };
    ParamInputComponent.prototype.ngOnInit = function () {
        // retrieve parameter from symbol
        this._paramDef = this.paramService.getParameter(this._paramSymbol);
    };
    // private getValue() {
    //     if (this._paramDef.isDefined)
    //         return this._paramDef.v;
    //     return "";
    //     // return this._uiValue.value;
    // }
    /**
     * fonction appelée lorsque l'utilisateur fait une saisie
     * @param event valeur du contrôle
     */
    ParamInputComponent.prototype.setValue = function (event) {
        this._fromUI = true;
        this._uiValue.value = event;
        this.log(this._uiValue.toString());
        return this.validateUIValue();
    };
    /**
     * fonction appelée lors d'un rafraîchissement de l'UI
     */
    ParamInputComponent.prototype.ngDoCheck = function () {
        this.log("ngDoCheck start : " + this.getSParam() + this.getSUIvalue() + this.getSfromUI());
        if (this._fromUI)
            this.updateMessage(this._uiValue);
        else {
            if (this._paramDef.isDefined) {
                this.updateMessage(new jalhyd_1.NumericalString(this._paramDef.v));
                this._uiValue.value = String(this._paramDef.v);
            }
            else
                this.updateMessage(this._uiValue);
        }
        this.log("ngDoCheck end : " + this.getSParam() + this.getSUIvalue());
        this._fromUI = false;
    };
    ParamInputComponent.prototype.updateMessage = function (v) {
        this.log("updateMessage start :" + this.getSParam() + this.getSfromUI() + this.getSUIvalue(v) + "  message=" + this._message);
        if (v.isNumerical) {
            this._message = undefined;
            try {
                this._paramDef.checkValue(v.numericalValue);
            }
            catch (e) {
                if (e instanceof jalhyd_1.ErrorMessage)
                    this._message = this.intlService.localizeErrorMessage(e);
                else
                    this._message = "invalid value";
            }
        }
        else {
            switch (this.intlService.lang) {
                case internationalisation_service_1.Language.FRENCH:
                    this._message = "Veuillez entrer une valeur numérique";
                    break;
                default:
                    this._message = "Please enter a numerical value";
            }
        }
        this.log("updateMessage end :" + this.getSParam() + this.getSfromUI() + this.getSUIvalue(v) + "  message=" + this._message);
    };
    ParamInputComponent.prototype.validateUIValue = function () {
        this.log("");
        this.log("validateValue start : val '" + this._uiValue.toString() + "'" + this.getSParam() + this.getSfromUI());
        var ok = this._uiValue.isNumerical;
        if (ok) {
            try {
                if (!this._paramDef.isDefined || this._paramDef.v != this._uiValue.numericalValue) {
                    this._paramDef.v = this._uiValue.numericalValue;
                    this.changeDetector.detectChanges(); // provoque une détection des changements dans les contrôles
                }
            }
            catch (e) {
                ok = false;
            }
        }
        if (!ok) {
            this.log("validateValue end : " + this.getSParam());
            var err = {
                rangeError: {
                    // given: val,
                    given: this._uiValue.toString(),
                    max: 4,
                    min: 0
                }
            };
            return err;
        }
        this.log("validateValue end : " + this.getSParam());
        return null;
    };
    ParamInputComponent.prototype.log = function (m) {
        // let t: number = new Date().getTime() - ParamInputComponent._startTime;
        //   console.log("ParamInputComponent(" + this._id + ") " + t + " : " + m);
    };
    /*
    //From ControlValueAccessor interface
    writeValue(value: any) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    }
    */
    ParamInputComponent.prototype.writeValue = function (value) {
        this.log("writeValue " + value);
    };
    ParamInputComponent.prototype.registerOnChange = function (fn) {
        this.propagateChange = fn;
    };
    ParamInputComponent.prototype.registerOnTouched = function () { };
    return ParamInputComponent;
}());
ParamInputComponent._idGen = 0; // A VIRER
__decorate([
    core_1.Input('symbol'),
    __metadata("design:type", String)
], ParamInputComponent.prototype, "_paramSymbol", void 0);
ParamInputComponent = ParamInputComponent_1 = __decorate([
    core_1.Component({
        selector: 'param-input[symbol]',
        /* OK
        <input placeholder="{{_paramDef.symbol}}" [ngModel]="_paramDef.v" (ngModelChange)="setValue($event)"/>
        <p *ngIf="_message">{{_message}}</p>
        */
        templateUrl: "./param-input.component.html",
        providers: [
            {
                provide: forms_1.NG_VALUE_ACCESSOR,
                useExisting: core_1.forwardRef(function () { return ParamInputComponent_1; }),
                multi: true
            },
            {
                provide: forms_1.NG_VALIDATORS,
                useExisting: core_1.forwardRef(function () { return ParamInputComponent_1; }),
                multi: true
            }
        ]
    }),
    __metadata("design:paramtypes", [param_service_1.ParamService, core_1.ChangeDetectorRef, internationalisation_service_1.InternationalisationService])
], ParamInputComponent);
exports.ParamInputComponent = ParamInputComponent;
var ParamInputComponent_1;
//# sourceMappingURL=param-input.component.js.map